@@ -81,10 +81,11 @@ from bpython import translations
 from bpython.translations import _
 
 from bpython import repl
+from bpython._py3compat import py3
 from bpython.pager import page
+from bpython import autocomplete
 import bpython.args
 
-py3 = sys.version_info[0] == 3
 if not py3:
     import inspect
 
@@ -116,6 +117,25 @@ def calculate_screen_lines(tokens, width, cursor=0):
     return lines
 
 
+class FakeStream(object):
+    """Provide a fake file object which calls functions on the interface
+    provided."""
+
+    def __init__(self, interface):
+        self.encoding = getpreferredencoding()
+        self.interface = interface
+
+    def write(self, s):
+        self.interface.write(s)
+
+    def writelines(self, l):
+        for s in l:
+            self.write(s)
+
+    def isatty(self):
+        # some third party (amongst them mercurial) depend on this
+        return True
+
 class FakeStdin(object):
     """Provide a fake stdin type for things like raw_input() etc."""
 
@@ -172,8 +192,8 @@ class FakeStdin(object):
                     (len(key) > 1 or unicodedata.category(key) == 'Cc')):
                     continue
                 sys.stdout.write(key)
-# Include the \n in the buffer - raw_input() seems to deal with trailing
-# linebreaks and will break if it gets an empty string.
+                # Include the \n in the buffer - raw_input() seems to deal with trailing
+                # linebreaks and will break if it gets an empty string.
                 buffer += key
         finally:
             curses.raw(False)
@@ -301,6 +321,7 @@ class CLIRepl(repl.Repl):
         self.list_win = newwin(get_colpair(config, 'background'), 1, 1, 1, 1)
         self.cpos = 0
         self.do_exit = False
+        self.exit_value = ()
         self.f_string = ''
         self.idle = idle
         self.in_hist = False
@@ -352,7 +373,7 @@ class CLIRepl(repl.Repl):
         self.clear_wrapped_lines()
 
         if not self.cpos:
-# I know the nested if blocks look nasty. :(
+            # I know the nested if blocks look nasty. :(
             if self.atbol() and delete_tabs:
                 n = len(self.s) % self.config.tab_length
                 if not n:
@@ -370,11 +391,11 @@ class CLIRepl(repl.Repl):
         self.rl_history.reset()
         pos = len(self.s) - self.cpos - 1
         deleted = []
-# First we delete any space to the left of the cursor.
+        # First we delete any space to the left of the cursor.
         while pos >= 0 and self.s[pos] == ' ':
             deleted.append(self.s[pos])
             pos -= self.bs()
-# Then we delete a full word.
+        # Then we delete a full word.
         while pos >= 0 and self.s[pos] != ' ':
             deleted.append(self.s[pos])
             pos -= self.bs()
@@ -407,6 +428,7 @@ class CLIRepl(repl.Repl):
             self.scr.clrtoeol()
 
     def complete(self, tab=False):
+        """Get Autcomplete list and window."""
         if self.paste_mode and self.list_win_visible:
             self.scr.touchwin()
 
@@ -416,6 +438,7 @@ class CLIRepl(repl.Repl):
         if self.list_win_visible and not self.config.auto_display_list:
             self.scr.touchwin()
             self.list_win_visible = False
+            self.matches_iter.update()
             return
 
         if self.config.auto_display_list or tab:
@@ -464,24 +487,23 @@ class CLIRepl(repl.Repl):
         """Return the current word, i.e. the (incomplete) word directly to the
         left of the cursor"""
 
-        if self.cpos:
-# I don't know if autocomplete should be disabled if the cursor
-# isn't at the end of the line, but that's what this does for now.
-            return
+        # I don't know if autocomplete should be disabled if the cursor
+        # isn't at the end of the line, but that's what this does for now.
+        if self.cpos: return
 
+        # look from right to left for a bad method character
         l = len(self.s)
+        is_method_char = lambda c: c.isalnum() or c in ('.', '_')
 
-        if (not self.s or
-            (not self.s[l - 1].isalnum() and
-             self.s[l - 1] not in ('.', '_'))):
+        if not self.s or not is_method_char(self.s[l-1]):
             return
 
-        i = 1
-        while i < l + 1:
-            if not self.s[-i].isalnum() and self.s[-i] not in ('.', '_'):
+        for i in range(1, l+1):
+            if not is_method_char(self.s[-i]):
+                i -= 1
                 break
-            i += 1
-        return self.s[-i + 1:]
+
+        return self.s[-i:]
 
     def delete(self):
         """Process a del"""
@@ -599,27 +621,30 @@ class CLIRepl(repl.Repl):
                     key = key.decode(getpreferredencoding())
                 self.scr.nodelay(False)
             except UnicodeDecodeError:
-# Yes, that actually kind of sucks, but I don't see another way to get
-# input right
+                # Yes, that actually kind of sucks, but I don't see another way to get
+                # input right
                 self.scr.nodelay(True)
             except curses.error:
-# I'm quite annoyed with the ambiguity of this exception handler. I previously
-# caught "curses.error, x" and accessed x.message and checked that it was "no
-# input", which seemed a crappy way of doing it. But then I ran it on a
-# different computer and the exception seems to have entirely different
-# attributes. So let's hope getkey() doesn't raise any other crazy curses
-# exceptions. :)
+                # I'm quite annoyed with the ambiguity of this exception handler. I previously
+                # caught "curses.error, x" and accessed x.message and checked that it was "no
+                # input", which seemed a crappy way of doing it. But then I ran it on a
+                # different computer and the exception seems to have entirely different
+                # attributes. So let's hope getkey() doesn't raise any other crazy curses
+                # exceptions. :)
                 self.scr.nodelay(False)
                 # XXX What to do here? Raise an exception?
                 if key:
                     return key
             else:
-                t = time.time()
-                self.paste_mode = (
-                    t - self.last_key_press <= self.config.paste_time
-                )
-                self.last_key_press = t
-                return key
+                if key != '\x00':
+                    t = time.time()
+                    self.paste_mode = (
+                        t - self.last_key_press <= self.config.paste_time
+                    )
+                    self.last_key_press = t
+                    return key
+                else:
+                    key = ''
             finally:
                 if self.idle:
                     self.idle(self)
@@ -646,6 +671,8 @@ class CLIRepl(repl.Repl):
         while True:
             key = self.get_key()
             if self.p_key(key) is None:
+                if self.config.cli_trim_prompts and self.s.startswith(">>> "):
+                    self.s = self.s[4:]
                 return self.s
 
     def home(self, refresh=True):
@@ -655,9 +682,6 @@ class CLIRepl(repl.Repl):
             self.scr.refresh()
         return True
 
-    def isatty(self):
-        return True
-
     def lf(self):
         """Process a linefeed character; it only needs to check the
         cursor position and move appropriately so it doesn't clear
@@ -1046,9 +1070,10 @@ class CLIRepl(repl.Repl):
         curses.raw(False)
         try:
             return repl.Repl.push(self, s, insert_into_history)
-        except SystemExit:
+        except SystemExit, e:
             # Avoid a traceback on e.g. quit()
             self.do_exit = True
+            self.exit_value = e.args
             return False
         finally:
             curses.raw(True)
@@ -1078,8 +1103,8 @@ class CLIRepl(repl.Repl):
         up/down to go back and forth (which has to be separate to the
         evaluation history, which will be truncated when undoing."""
 
-# Use our own helper function because Python's will use real stdin and
-# stdout instead of our wrapped
+        # Use our own helper function because Python's will use real stdin and
+        # stdout instead of our wrapped
         self.push('from bpython._internal import _help as help\n', False)
 
         self.iy, self.ix = self.scr.getyx()
@@ -1098,7 +1123,7 @@ class CLIRepl(repl.Repl):
 
             self.scr.redrawwin()
             if self.do_exit:
-                return
+                return self.exit_value
 
             self.history.append(inp)
             self.s_hist[-1] += self.f_string
@@ -1111,6 +1136,7 @@ class CLIRepl(repl.Repl):
             if not more:
                 self.prev_block_finished = stdout_position
                 self.s = ''
+        return self.exit_value
 
     def reprint_line(self, lineno, tokens):
         """Helper function for paren highlighting: Reprint line at offset
@@ -1174,8 +1200,8 @@ class CLIRepl(repl.Repl):
                 self.stdout_hist += line.encode(getpreferredencoding()) + '\n'
             self.print_line(line)
             self.s_hist[-1] += self.f_string
-# I decided it was easier to just do this manually
-# than to make the print_line and history stuff more flexible.
+            # I decided it was easier to just do this manually
+            # than to make the print_line and history stuff more flexible.
             self.scr.addstr('\n')
             more = self.push(line)
             self.prompt(more)
@@ -1266,7 +1292,7 @@ class CLIRepl(repl.Repl):
             return True
 
         if items:
-# visible items (we'll append until we can't fit any more in)
+            # visible items (we'll append until we can't fit any more in)
             v_items = [items[0][:max_w - 3]]
             lsize()
         else:
@@ -1339,9 +1365,9 @@ class CLIRepl(repl.Repl):
                 docstring_string = docstring_string.encode(encoding, 'ignore')
             self.list_win.addstr('\n' + docstring_string,
                                  get_colpair(self.config, 'comment'))
-# XXX: After all the trouble I had with sizing the list box (I'm not very good
-# at that type of thing) I decided to do this bit of tidying up here just to
-# make sure there's no unnececessary blank lines, it makes things look nicer.
+            # XXX: After all the trouble I had with sizing the list box (I'm not very good
+            # at that type of thing) I decided to do this bit of tidying up here just to
+            # make sure there's no unnececessary blank lines, it makes things look nicer.
 
         y = self.list_win.getyx()[0]
         self.list_win.resize(y + 2, w)
@@ -1354,8 +1380,8 @@ class CLIRepl(repl.Repl):
         self.scr.cursyncup()
         self.scr.noutrefresh()
 
-# This looks a little odd, but I can't figure a better way to stick the cursor
-# back where it belongs (refreshing the window hides the list_win)
+        # This looks a little odd, but I can't figure a better way to stick the cursor
+        # back where it belongs (refreshing the window hides the list_win)
 
         self.scr.move(*self.scr.getyx())
         self.list_win.refresh()
@@ -1377,13 +1403,20 @@ class CLIRepl(repl.Repl):
             os.kill(os.getpid(), signal.SIGSTOP)
 
     def tab(self, back=False):
-        """Process the tab key being hit. If there's only whitespace
+        """Process the tab key being hit. 
+
+        If there's only whitespace
         in the line or the line is blank then process a normal tab,
         otherwise attempt to autocomplete to the best match of possible
         choices in the match list.
-        If `back` is True, walk backwards through the list of suggestions
-        and don't indent if there are only whitespace in the line."""
 
+        If `back` is True, walk backwards through the list of suggestions
+        and don't indent if there are only whitespace in the line.
+        """
+
+        mode = self.config.autocomplete_mode
+
+        # 1. check if we should add a tab character
         if self.atbol() and not back:
             x_pos = len(self.s) - self.cpos
             num_spaces = x_pos % self.config.tab_length
@@ -1394,6 +1427,7 @@ class CLIRepl(repl.Repl):
             self.print_line(self.s)
             return True
 
+        # 2. get the current word
         if not self.matches_iter:
             self.complete(tab=True)
             if not self.config.auto_display_list and not self.list_win_visible:
@@ -1405,25 +1439,38 @@ class CLIRepl(repl.Repl):
         else:
             cw = self.matches_iter.current_word
 
-        b = os.path.commonprefix(self.matches)
-        if b:
-            self.s += b[len(cw):]
-            expanded = bool(b[len(cw):])
+        # 3. check to see if we can expand the current word
+        cseq = None
+        if mode == autocomplete.SUBSTRING:
+            if all([len(match.split(cw)) == 2 for match in self.matches]):
+                seq = [cw + match.split(cw)[1] for match in self.matches]
+                cseq = os.path.commonprefix(seq)
+        else:
+            seq = self.matches
+            cseq = os.path.commonprefix(seq)
+
+        if cseq and mode != autocomplete.FUZZY:
+            expanded_string = cseq[len(cw):]
+            self.s += expanded_string
+            expanded = bool(expanded_string)
             self.print_line(self.s)
             if len(self.matches) == 1 and self.config.auto_display_list:
                 self.scr.touchwin()
             if expanded:
-                self.matches_iter.update(b, self.matches)
+                self.matches_iter.update(cseq, self.matches)
         else:
             expanded = False
 
+        # 4. swap current word for a match list item
         if not expanded and self.matches:
+            # reset s if this is the nth result
             if self.matches_iter:
                 self.s = self.s[:-len(self.matches_iter.current())] + cw
-            if back:
-                current_match = self.matches_iter.previous()
-            else:
-                current_match = self.matches_iter.next()
+
+            current_match = back and self.matches_iter.previous() \
+                                  or self.matches_iter.next()
+
+            # update s with the new match
             if current_match:
                 try:
                     self.show_list(self.matches, self.argspec, current_match)
@@ -1433,7 +1480,12 @@ class CLIRepl(repl.Repl):
                     # using it.
                     self.list_win.border()
                     self.list_win.refresh()
-                self.s += current_match[len(cw):]
+
+                if self.config.autocomplete_mode == autocomplete.SIMPLE:
+                    self.s += current_match[len(cw):]
+                else:
+                    self.s = self.s[:-len(cw)] + current_match
+
                 self.print_line(self.s, True)
         return True
 
@@ -1610,7 +1662,7 @@ class Statusbar(object):
 def init_wins(scr, config):
     """Initialise the two windows (the main repl interface and the little
     status bar at the bottom with some stuff in it)"""
-#TODO: Document better what stuff is on the status bar.
+    #TODO: Document better what stuff is on the status bar.
 
     background = get_colpair(config, 'background')
     h, w = gethw()
@@ -1618,8 +1670,8 @@ def init_wins(scr, config):
     main_win = newwin(background, h - 1, w, 0, 0)
     main_win.scrollok(True)
     main_win.keypad(1)
-# Thanks to Angus Gibson for pointing out this missing line which was causing
-# problems that needed dirty hackery to fix. :)
+    # Thanks to Angus Gibson for pointing out this missing line which was causing
+    # problems that needed dirty hackery to fix. :)
 
     statusbar = Statusbar(scr, main_win, background, config,
         _(" <%s> Rewind  <%s> Save  <%s> Pastebin "
@@ -1638,7 +1690,7 @@ def sigwinch(unused_scr):
 def sigcont(unused_scr):
     sigwinch(unused_scr)
     # Forces the redraw
-    curses.ungetch('')
+    curses.ungetch('\x00')
 
 def gethw():
     """I found this code on a usenet post, and snipped out the bit I needed,
@@ -1657,7 +1709,6 @@ def gethw():
     """
 
     if platform.system() != 'Windows':
-        import struct
         h, w = struct.unpack(
             "hhhh",
             fcntl.ioctl(sys.__stdout__, termios.TIOCGWINSZ, "\000" * 8))[0:2]
@@ -1673,7 +1724,6 @@ def gethw():
         res = windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
 
         if res:
-            import struct
             (bufx, bufy, curx, cury, wattr,
              left, top, right, bottom, maxx, maxy) = struct.unpack("hhhhHhhhhhh", csbi.raw)
             sizex = right - left + 1
@@ -1697,7 +1747,10 @@ def idle(caller):
         caller.scr.nodelay(True)
         key = caller.scr.getch()
         caller.scr.nodelay(False)
-        curses.ungetch(key)
+        if key != -1:
+            curses.ungetch(key)
+        else:
+            curses.ungetch('\x00')
     caller.statusbar.check()
     caller.check()
 
@@ -1711,7 +1764,7 @@ def do_resize(caller):
     global DO_RESIZE
     h, w = gethw()
     if not h:
-# Hopefully this shouldn't happen. :)
+    # Hopefully this shouldn't happen. :)
         return
 
     curses.endwin()
@@ -1721,7 +1774,7 @@ def do_resize(caller):
     DO_RESIZE = False
 
     caller.resize()
-# The list win resizes itself every time it appears so no need to do it here.
+    # The list win resizes itself every time it appears so no need to do it here.
 
 
 class FakeDict(object):
@@ -1777,6 +1830,9 @@ def main_curses(scr, args, config, interactive=True, locals_=None,
     I've tried to keep it well factored but it needs some
     tidying up, especially in separating the curses stuff
     from the rest of the repl.
+
+    Returns a tuple (exit value, output), where exit value is a tuple
+    with arguments passed to SystemExit.
     """
     global stdscr
     global DO_RESIZE
@@ -1814,8 +1870,8 @@ def main_curses(scr, args, config, interactive=True, locals_=None,
     clirepl._C = cols
 
     sys.stdin = FakeStdin(clirepl)
-    sys.stdout = clirepl
-    sys.stderr = clirepl
+    sys.stdout = FakeStream(clirepl)
+    sys.stderr = FakeStream(clirepl)
 
     if args:
         bpython.args.exec_code(interpreter, args)
@@ -1829,7 +1885,7 @@ def main_curses(scr, args, config, interactive=True, locals_=None,
     if banner is not None:
         clirepl.write(banner)
         clirepl.write('\n')
-    clirepl.repl()
+    exit_value = clirepl.repl()
 
     main_win.erase()
     main_win.refresh()
@@ -1842,11 +1898,10 @@ def main_curses(scr, args, config, interactive=True, locals_=None,
         signal.signal(signal.SIGWINCH, old_sigwinch_handler)
         signal.signal(signal.SIGCONT, old_sigcont_handler)
 
-    return clirepl.getstdout()
+    return (exit_value, clirepl.getstdout())
 
 
 def main(args=None, locals_=None, banner=None):
-    locale.setlocale(locale.LC_ALL, "")
     translations.init()
 
 
@@ -1858,22 +1913,23 @@ def main(args=None, locals_=None, banner=None):
     orig_stderr = sys.stderr
 
     try:
-        o = curses_wrapper(main_curses, exec_args, config,
-                           options.interactive, locals_,
-                           banner=banner)
+        (exit_value, output) = curses_wrapper(
+            main_curses, exec_args, config, options.interactive, locals_,
+            banner=banner)
     finally:
         sys.stdin = orig_stdin
         sys.stderr = orig_stderr
         sys.stdout = orig_stdout
 
-# Fake stdout data so everything's still visible after exiting
+    # Fake stdout data so everything's still visible after exiting
     if config.flush_output and not options.quiet:
-        sys.stdout.write(o)
-    sys.stdout.flush()
-
+        sys.stdout.write(output)
+    if hasattr(sys.stdout, 'flush'):
+        sys.stdout.flush()
+    return repl.extract_exit_value(exit_value)
 
 if __name__ == '__main__':
     from bpython.cli import main
-    main()
+    sys.exit(main())
 
 # vim: sw=4 ts=4 sts=4 ai et
