@@ -4,7 +4,7 @@ import sys
 from ConfigParser import ConfigParser
 from itertools import chain
 from bpython.keys import cli_key_dispatch as key_dispatch
-
+from bpython.autocomplete import SIMPLE as default_completion
 
 MAGIC_METHODS = ", ".join("__%s__" % s for s in [
     "init", "repr", "str", "lt", "le", "eq", "ne", "gt", "ge", "cmp", "hash",
@@ -56,6 +56,7 @@ def loadini(struct, configfile):
             'color_scheme': 'default',
             'complete_magic_methods' : True,
             'magic_methods' : MAGIC_METHODS,
+            'autocomplete_mode': default_completion,
             'dedent_after': 1,
             'flush_output': True,
             'highlight_show_source': True,
@@ -68,7 +69,9 @@ def loadini(struct, configfile):
             'pastebin_confirm': True,
             'pastebin_private': False,
             'pastebin_url': 'http://bpaste.net/xmlrpc/',
+            'pastebin_private': True,
             'pastebin_show_url': 'http://bpaste.net/show/$paste_id/',
+            'pastebin_helper': '',
         },
         'keyboard': {
             'clear_line': 'C-u',
@@ -89,6 +92,7 @@ def loadini(struct, configfile):
             'yank_from_buffer': 'C-y'},
         'cli': {
             'suggestion_width': 0.8,
+            'trim_prompts': False,
         },
         'gtk': {
             'font': 'monospace 10',
@@ -134,15 +138,20 @@ def loadini(struct, configfile):
     struct.pastebin_confirm = config.getboolean('general', 'pastebin_confirm')
     struct.pastebin_private = config.getboolean('general', 'pastebin_private')
     struct.pastebin_url = config.get('general', 'pastebin_url')
+    struct.pastebin_private = config.get('general', 'pastebin_private')
     struct.pastebin_show_url = config.get('general', 'pastebin_show_url')
+    struct.pastebin_helper = config.get('general', 'pastebin_helper')
 
     struct.cli_suggestion_width = config.getfloat('cli',
                                                   'suggestion_width')
+    struct.cli_trim_prompts = config.getboolean('cli',
+                                                  'trim_prompts')
 
     struct.complete_magic_methods = config.getboolean('general',
                                                       'complete_magic_methods')
     methods = config.get('general', 'magic_methods')
     struct.magic_methods = [meth.strip() for meth in methods.split(",")]
+    struct.autocomplete_mode = config.get('general', 'autocomplete_mode')
 
     struct.gtk_font = config.get('gtk', 'font')
 
