@@ -24,27 +24,32 @@
 from __future__ import with_statement
 import code
 import codecs
+import errno
 import inspect
 import os
 import pydoc
-import re
-import rlcompleter
+import subprocess
 import sys
 import textwrap
 import traceback
+import unicodedata
 from glob import glob
 from itertools import takewhile
 from locale import getpreferredencoding
 from socket import error as SocketError
 from string import Template
 from urllib import quote as urlquote
+from urlparse import urlparse
 from xmlrpclib import ServerProxy, Error as XMLRPCError
 
-from pygments.lexers import PythonLexer
 from pygments.token import Token
 
 from bpython import importcompletion, inspection
+from bpython._py3compat import PythonLexer, py3
 from bpython.formatter import Parenthesis
+from bpython.translations import _
+from bpython.autocomplete import Autocomplete
+
 
 # Needed for special handling of __abstractmethods__
 # abc only exists since 2.6, so check both that it exists and that it's
@@ -56,8 +61,6 @@ try:
 except (ImportError, AttributeError):
     has_abc = False
 
-py3 = sys.version_info[0] == 3
-
 
 class Interpreter(code.InteractiveInterpreter):
 
@@ -72,7 +75,7 @@ class Interpreter(code.InteractiveInterpreter):
 
         self.encoding = encoding or sys.getdefaultencoding()
         self.syntaxerror_callback = None
-# Unfortunately code.InteractiveInterpreter is a classic class, so no super()
+        # Unfortunately code.InteractiveInterpreter is a classic class, so no super()
         code.InteractiveInterpreter.__init__(self, locals)
 
     if not py3:
@@ -373,13 +376,7 @@ class Repl(object):
         self.s_hist = []
         self.history = []
         self.evaluating = False
-# Use the interpreter's namespace only for the readline stuff:
-        self.completer = rlcompleter.Completer(self.interp.locals)
-        self.completer.attr_matches = self.attr_matches
-        # Gna, Py 2.6's rlcompleter searches for __call__ inside the
-        # instance instead of the type, so we monkeypatch to prevent
-        # side-effects (__getattr__/__getattribute__)
-        self.completer._callable_postfix = self._callable_postfix
+        self.completer = Autocomplete(self.interp.locals, config)
         self.matches = []
         self.matches_iter = MatchesIterator()
         self.argspec = None
@@ -417,53 +414,8 @@ class Repl(object):
                 else:
                     self.interp.runsource(f.read(), filename, 'exec', encode=False)
 
-    def attr_matches(self, text):
-        """Taken from rlcompleter.py and bent to my will."""
-
-        m = re.match(r"(\w+(\.\w+)*)\.(\w*)", text)
-        if not m:
-            return []
-
-        expr, attr = m.group(1, 3)
-        if expr.isdigit():
-            # Special case: float literal, using attrs here will result in
-            # a SyntaxError
-            return []
-        obj = eval(expr, self.interp.locals)
-        with inspection.AttrCleaner(obj):
-            matches = self.attr_lookup(obj, expr, attr)
-        return matches
-
-    def attr_lookup(self, obj, expr, attr):
-        """Second half of original attr_matches method factored out so it can
-        be wrapped in a safe try/finally block in case anything bad happens to
-        restore the original __getattribute__ method."""
-        words = dir(obj)
-        if hasattr(obj, '__class__'):
-            words.append('__class__')
-            words = words + rlcompleter.get_class_members(obj.__class__)
-            if has_abc and not isinstance(obj.__class__, abc.ABCMeta):
-                try:
-                    words.remove('__abstractmethods__')
-                except ValueError:
-                    pass
-
-        matches = []
-        n = len(attr)
-        for word in words:
-            if word[:n] == attr and word != "__builtins__":
-                matches.append("%s.%s" % (expr, word))
-        return matches
-
-    def _callable_postfix(self, value, word):
-        """rlcompleter's _callable_postfix done right."""
-        with inspection.AttrCleaner(value):
-            if inspection.is_callable(value):
-                word += '('
-        return word
-
     def current_string(self, concatenate=False):
-        """Return the current string."""
+        """If the line ends in a string get it, otherwise return ''"""
         tokens = self.tokenize(self.current_line())
         string_tokens = list(takewhile(token_is_any_of([Token.String,
                                                         Token.Text]),
@@ -525,13 +477,17 @@ class Repl(object):
                             stack[-1][1] += 1
                         except TypeError:
                             stack[-1][1] = ''
+                        stack[-1][0] = ''
                     elif value == ':' and stack[-1][2] == 'lambda':
                         stack.pop()
+                    else:
+                        stack[-1][0] = ''
                 elif (token is Token.Name or token in Token.Name.subtypes or
                       token is Token.Operator and value == '.'):
                     stack[-1][0] += value
                 elif token is Token.Operator and value == '=':
                     stack[-1][1] = stack[-1][0]
+                    stack[-1][0] = ''
                 elif token is Token.Keyword and value == 'lambda':
                     stack.append(['', 0, value])
                 else:
@@ -640,10 +596,10 @@ class Repl(object):
             try:
                 self.completer.complete(cw, 0)
             except Exception:
-# This sucks, but it's either that or list all the exceptions that could
-# possibly be raised here, so if anyone wants to do that, feel free to send me
-# a patch. XXX: Make sure you raise here if you're debugging the completion
-# stuff !
+                # This sucks, but it's either that or list all the exceptions that could
+                # possibly be raised here, so if anyone wants to do that, feel free to send me
+                # a patch. XXX: Make sure you raise here if you're debugging the completion
+                # stuff !
                 e = True
             else:
                 matches = self.completer.matches
@@ -660,7 +616,7 @@ class Repl(object):
                 matches.extend(name + '=' for name in self.argspec[1][4]
                                if name.startswith(cw))
 
-# unless the first character is a _ filter out all attributes starting with a _
+        # unless the first character is a _ filter out all attributes starting with a _
         if not e and not cw.split('.')[-1].startswith('_'):
             matches = [match for match in matches
                        if not match.split('.')[-1].startswith('_')]
@@ -671,7 +627,7 @@ class Repl(object):
             if not self.argspec:
                 return False
         else:
-# remove duplicates
+            # remove duplicates
             self.matches = sorted(set(matches))
 
 
@@ -767,39 +723,91 @@ class Repl(object):
             s = self.getstdout()
 
         if (self.config.pastebin_confirm and
-            not self.interact.confirm("Pastebin buffer? (y/N) ")):
-            self.interact.notify("Pastebin aborted")
+            not self.interact.confirm(_("Pastebin buffer? (y/N) "))):
+            self.interact.notify(_("Pastebin aborted"))
             return
         return self.do_pastebin(s)
 
     def do_pastebin(self, s):
         """Actually perform the upload."""
+        if s == self.prev_pastebin_content:
+            self.interact.notify(_('Duplicate pastebin. Previous URL: %s') %
+                                  (self.prev_pastebin_url, ))
+            return self.prev_pastebin_url
+
+        if self.config.pastebin_helper:
+            return self.do_pastebin_helper(s)
+        else:
+            return self.do_pastebin_xmlrpc(s)
+
+    def do_pastebin_xmlrpc(self, s):
+        """Upload to pastebin via XML-RPC."""
         try:
             pasteservice = ServerProxy(self.config.pastebin_url)
         except IOError, e:
-            self.interact.notify("Pastebin error for URL '%s': %s" %
+            self.interact.notify(_("Pastebin error for URL '%s': %s") %
                                  (self.config.pastebin_url, str(e)))
             return
 
-        if s == self.prev_pastebin_content:
-            self.interact.notify('Duplicate pastebin. Previous URL: ' +
-                                  self.prev_pastebin_url)
-            return self.prev_pastebin_url
+        self.interact.notify(_('Posting data to pastebin...'))
+        try:
+            paste_id = pasteservice.pastes.newPaste('pycon', s, '', '', '',
+                   self.config.pastebin_private)
+        except (SocketError, XMLRPCError), e:
+            self.interact.notify(_('Upload failed: %s') % (str(e), ) )
+            return
 
         self.prev_pastebin_content = s
 
-        self.interact.notify('Posting data to pastebin...')
-        try:
-            paste_id = pasteservice.pastes.newPaste('pycon', s)
-        except (SocketError, XMLRPCError), e:
-            self.interact.notify('Upload failed: %s' % (str(e), ) )
-            return
-
         paste_url_template = Template(self.config.pastebin_show_url)
         paste_id = urlquote(paste_id)
         paste_url = paste_url_template.safe_substitute(paste_id=paste_id)
         self.prev_pastebin_url = paste_url
-        self.interact.notify('Pastebin URL: %s' % (paste_url, ), 10)
+        self.interact.notify(_('Pastebin URL: %s') % (paste_url, ), 10)
+        return paste_url
+
+    def do_pastebin_helper(self, s):
+        """Call out to helper program for pastebin upload."""
+        self.interact.notify(_('Posting data to pastebin...'))
+
+        try:
+            helper = subprocess.Popen('',
+                                      executable=self.config.pastebin_helper,
+                                      stdin=subprocess.PIPE,
+                                      stdout=subprocess.PIPE)
+            helper.stdin.write(s.encode(getpreferredencoding()))
+            output = helper.communicate()[0].decode(getpreferredencoding())
+            paste_url = output.split()[0]
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                self.interact.notify(_('Upload failed: '
+                                       'Helper program not found.'))
+            else:
+                self.interact.notify(_('Upload failed: '
+                                       'Helper program could not be run.'))
+            return
+
+        if helper.returncode != 0:
+            self.interact.notify(_('Upload failed: '
+                                   'Helper program returned non-zero exit '
+                                   'status %s.' % (helper.returncode, )))
+            return
+
+        if not paste_url:
+            self.interact.notify(_('Upload failed: '
+                                   'No output from helper program.'))
+            return
+        else:
+            parsed_url = urlparse(paste_url)
+            if (not parsed_url.scheme
+                or any(unicodedata.category(c) == 'Cc' for c in paste_url)):
+                self.interact.notify(_("Upload failed: "
+                                       "Failed to recognize the helper "
+                                       "program's output as an URL."))
+                return
+
+        self.prev_pastebin_content = s
+        self.interact.notify(_('Pastebin URL: %s') % (paste_url, ), 10)
         return paste_url
 
     def push(self, s, insert_into_history=True):
@@ -965,6 +973,9 @@ def next_indentation(line, tab_length):
     indentation = (len(line) - len(line.lstrip(' '))) // tab_length
     if line.rstrip().endswith(':'):
         indentation += 1
+    elif indentation >= 1:
+        if line.lstrip().startswith(('return', 'pass', 'raise', 'yield')):
+            indentation -= 1
     return indentation
 
 
@@ -1016,3 +1027,14 @@ def token_is_any_of(token_types):
         return any(check(token) for check in is_token_types)
 
     return token_is_any_of
+
+def extract_exit_value(args):
+    """Given the arguments passed to `SystemExit`, return the value that
+    should be passed to `sys.exit`.
+    """
+    if len(args) == 0:
+        return None
+    elif len(args) == 1:
+        return args[0]
+    else:
+        return args
