@@ -46,6 +46,7 @@ from optparse import Option
 from pygments.token import Token
 
 from bpython import args as bpargs, repl, translations
+from bpython._py3compat import py3
 from bpython.formatter import theme_map
 from bpython.importcompletion import find_coroutine
 from bpython.translations import _
@@ -54,7 +55,6 @@ from bpython.keys import urwid_key_dispatch as key_dispatch
 
 import urwid
 
-py3 = sys.version_info[0] == 3
 if not py3:
     import inspect
 
@@ -113,7 +113,10 @@ else:
             return EvalProtocol(self.repl)
 
 
-if urwid.VERSION < (1, 0, 0):
+# If Twisted is not available urwid has no TwistedEventLoop attribute.
+# Code below will try to import reactor before using TwistedEventLoop.
+# I assume TwistedEventLoop will be available if that import succeeds.
+if urwid.VERSION < (1, 0, 0) and hasattr(urwid, 'TwistedEventLoop'):
     class TwistedEventLoop(urwid.TwistedEventLoop):
 
         """TwistedEventLoop modified to properly stop the reactor.
@@ -144,7 +147,7 @@ if urwid.VERSION < (1, 0, 0):
                     self.reactor.crash()
             return wrapper
 else:
-    TwistedEventLoop = urwid.TwistedEventLoop
+    TwistedEventLoop = getattr(urwid, 'TwistedEventLoop', None)
 
 
 class StatusbarEdit(urwid.Edit):
@@ -404,6 +407,27 @@ class BPythonEdit(urwid.Edit):
                 self.edit_pos = 0
             elif urwid.command_map[key] == 'cursor max right':
                 self.edit_pos = len(self.get_edit_text())
+            elif urwid.command_map[key] == 'clear word':
+                # ^w
+                if self.edit_pos == 0:
+                    return
+                line = self.get_edit_text()
+                # delete any space left of the cursor
+                p = len(line[:self.edit_pos].strip())
+                line = line[:p] + line[self.edit_pos:]
+                # delete a full word
+                np = line.rfind(' ', 0, p)
+                if np == -1:
+                    line = line[p:]
+                    np = 0
+                else:
+                    line = line[:np] + line[p:]
+                self.set_edit_text(line)
+                self.edit_pos = np
+            elif urwid.command_map[key] == 'clear line':
+                line = self.get_edit_text()
+                self.set_edit_text(line[self.edit_pos:])
+                self.edit_pos = 0
             elif key == 'backspace':
                 line = self.get_edit_text()
                 cpos = len(line) - self.edit_pos
@@ -600,6 +624,9 @@ class URWIDRepl(repl.Repl):
         self.current_output = None
         self._completion_update_suppressed = False
 
+        # Bulletproof: this is a value extract_exit_value accepts.
+        self.exit_value = ()
+
         load_urwid_command_map(config)
 
     # Subclasses of Repl need to implement echo, current_line, cw
@@ -840,8 +867,8 @@ class URWIDRepl(repl.Repl):
                 self.stdout_hist += line.encode(locale.getpreferredencoding()) + '\n'
             self.print_line(line)
             self.s_hist[-1] += self.f_string
-# I decided it was easier to just do this manually
-# than to make the print_line and history stuff more flexible.
+            # I decided it was easier to just do this manually
+            # than to make the print_line and history stuff more flexible.
             self.scr.addstr('\n')
             more = self.push(line)
             self.prompt(more)
@@ -892,7 +919,8 @@ class URWIDRepl(repl.Repl):
         # Pretty blindly adapted from bpython.cli
         try:
             return repl.Repl.push(self, s, insert_into_history)
-        except SystemExit:
+        except SystemExit, e:
+            self.exit_value = e.args
             raise urwid.ExitMainLoop()
         except KeyboardInterrupt:
             # KeyboardInterrupt happened between the except block around
@@ -1082,15 +1110,18 @@ def main(args=None, locals_=None, banner=None):
     # TODO: maybe support displays other than raw_display?
     config, options, exec_args = bpargs.parse(args, (
             'Urwid options', None, [
+                Option('--twisted', '-T', action='store_true',
+                       help=_('Run twisted reactor.')),
                 Option('--reactor', '-r',
-                       help='Run a reactor (see --help-reactors)'),
+                       help=_('Select specific reactor (see --help-reactors). '
+                       'Implies --twisted.')),
                 Option('--help-reactors', action='store_true',
-                       help='List available reactors for -r'),
+                       help=_('List available reactors for -r.')),
                 Option('--plugin', '-p',
-                       help='twistd plugin to run (use twistd for a list). '
-                       'Use "--" to pass further options to the plugin.'),
+                       help=_('twistd plugin to run (use twistd for a list). '
+                       'Use "--" to pass further options to the plugin.')),
                 Option('--server', '-s', type='int',
-                       help='Port to run an eval server on (forces Twisted)'),
+                       help=_('Port to run an eval server on (forces Twisted).')),
                 ]))
 
     if options.help_reactors:
@@ -1112,8 +1143,8 @@ def main(args=None, locals_=None, banner=None):
             ('bold ' + name, color + ',bold', background, monochrome)
             for name, color, background, monochrome in palette])
 
-    if (options.server or options.plugin) and not options.reactor:
-        options.reactor = 'select'
+    if options.server or options.plugin:
+        options.twisted = True
 
     if options.reactor:
         try:
@@ -1132,6 +1163,14 @@ def main(args=None, locals_=None, banner=None):
                     options.reactor,))
             return
         event_loop = TwistedEventLoop(reactor)
+    elif options.twisted:
+        try:
+            from twisted.internet import reactor
+        except ImportError:
+            sys.stderr.write('No reactors are available. Please install '
+                'twisted for reactor support.\n')
+            return
+        event_loop = TwistedEventLoop(reactor)
     else:
         # None, not urwid.SelectEventLoop(), to work with
         # screens that do not support external event loops.
@@ -1264,7 +1303,9 @@ def main(args=None, locals_=None, banner=None):
 
     if config.flush_output and not options.quiet:
         sys.stdout.write(myrepl.getstdout())
-    sys.stdout.flush()
+    if hasattr(sys.stdout, "flush"):
+        sys.stdout.flush()
+    return repl.extract_exit_value(myrepl.exit_value)
 
 def load_urwid_command_map(config):
     urwid.command_map[key_dispatch[config.up_one_line_key]] = 'cursor up'
@@ -1272,12 +1313,15 @@ def load_urwid_command_map(config):
     urwid.command_map[key_dispatch['C-a']] = 'cursor max left'
     urwid.command_map[key_dispatch['C-e']] = 'cursor max right'
     urwid.command_map[key_dispatch[config.pastebin_key]] = 'pastebin'
+    urwid.command_map[key_dispatch['C-f']] = 'cursor right'
+    urwid.command_map[key_dispatch['C-b']] = 'cursor left'
+    urwid.command_map[key_dispatch['C-d']] = 'delete'
+    urwid.command_map[key_dispatch[config.clear_word_key]] = 'clear word'
+    urwid.command_map[key_dispatch[config.clear_line_key]] = 'clear line'
+
 """
-            'clear_line': 'C-u',
             'clear_screen': 'C-l',
-            'clear_word': 'C-w',
             'cut_to_buffer': 'C-k',
-            'delete': 'C-d',
             'down_one_line': 'C-n',
             'exit': '',
             'last_output': 'F9',
@@ -1290,4 +1334,4 @@ def load_urwid_command_map(config):
             'yank_from_buffer': 'C-y'},
 """
 if __name__ == '__main__':
-    main()
+    sys.exit(main())
